///
//  CircularGauge.m
//  HA7 logger mini
//  Copyright(C)2012-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.02.08 v0.0  First cording
//    2012.02.12 v0.2  Commit
//    2013.04.27 v0.4  iOS
//    2013.05.06 v0.4  Smooth move
//    2015.03.10 v0.5  scale1 and 2
//

#import "CircularGauge.h"
#import <CoreGraphics/CoreGraphics.h>

@implementation CircularGauge
@synthesize val;
@synthesize val_t;
@synthesize val_l;
@synthesize val_h;
@synthesize val_w;
@synthesize val_c;
@synthesize val_peak1;
@synthesize val_peak2;
@synthesize val_s1;                         // ADD 2015.03.10
@synthesize val_s2;                         // ADD 2015.03.10
@synthesize val_low;
@synthesize val_high;
@synthesize val_warning;
@synthesize val_critical;
@synthesize val_scale1;                     // ADD 2015.03.10
@synthesize val_scale2;                     // ADD 2015.03.10

- (void)awakeFromNib
{
	val_h  = [val_high floatValue];
	val_l  = [val_low floatValue];
	val_w  = [val_warning floatValue];
	val_c  = [val_critical floatValue];
    val_s1 = [val_scale1 floatValue];       // ADD 2015.03.10
    val_s2 = [val_scale2 floatValue];       // ADD 2015.03.10
	val    = val_h;
	val_t  = 0;
	val_peak1 = 0;
	val_peak2 = 0;
 	self.updateTimer = [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(refreshView:) userInfo:nil repeats:YES];
}

CGColorRef createDeviceRGBColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	CGColorSpaceRef rgb = CGColorSpaceCreateDeviceRGB();
	CGFloat comps[] = {r, g, b, a};
	CGColorRef color = CGColorCreate(rgb, comps);
	CGColorSpaceRelease(rgb);
	return color;
}

void drawNeedle(CGContextRef context, CGFloat x, CGFloat y, CGFloat r1, CGFloat r2,
                float ratio, CGFloat lineWidth, CGColorRef lineColor)
{
    const float     pi = 3.1415;
	float           rad;
	CGFloat         x1,y1;
	CGFloat         x2,y2;

	rad = pi * ratio + pi;
	x1 = cos(rad) * r1 + x;
	y1 = sin(rad) * r1 + y;
	x2 = cos(rad) * r2 + x;
	y2 = sin(rad) * r2 + y;
    CGContextSetLineWidth(context, lineWidth);
    CGContextSetStrokeColorWithColor(context, lineColor);
    CGContextMoveToPoint(context, x1, y1);
    CGContextAddLineToPoint(context, x2, y2);
	CGContextStrokePath(context);
    return;
}

//- (void)drawRect:(NSRect)rect
- (void)drawRect:(CGRect)rect
{
	CGContextRef    c            = UIGraphicsGetCurrentContext();
    const CGFloat   needleWidth1 = 1.0;
    const CGFloat   needleWidth2 = 1.0;
    const CGFloat   needleWidth3 = 5.0;
    const CGFloat   needleWidth4 = 2.0;         // ADD 2015.03.10  Large scale
    const CGFloat   needleWidth5 = 1.0;         // ADD 2015.03.10  Small scale
    CGColorRef      needleColor1 = createDeviceRGBColor(0.5, 0.5, 0.5, 1.0);
    CGColorRef      needleColor2 = createDeviceRGBColor(0.5, 0.5, 0.5, 1.0);
    CGColorRef      needleColor3;
	CGFloat         cx,cy;
	CGFloat         r1,r2;
    CGFloat         r4,r5;                      // ADD 2015.03.10
	float           ratio;
    float           v;                          // ADD 2015.03.10

	if (val > val_c) {
        needleColor3 = createDeviceRGBColor(1.0, 0.0, 0.0, 1.0);
	} else if (val > val_w) {
        needleColor3 = createDeviceRGBColor(1.0, 1.0, 0.0, 1.0);
	} else {
        needleColor3 = createDeviceRGBColor(0.0, 1.0, 0.0, 1.0);
	}

	cx = CGRectGetMidX(rect);
	cy = cx;
	r1 = CGRectGetWidth(rect) * 0.5;
	r2 = CGRectGetWidth(rect) * 0.2;
    r4 = CGRectGetWidth(rect) * 0.4;            // ADD 2015.03.10
    r5 = CGRectGetWidth(rect) * 0.47;           // ADD 2015.03.10
    
    // Scale 2                                  // ADD START 2015.03.10
    if (val_s2 > 0.0) {
        for(v = val_l; v <= val_h; v += val_s2){
            ratio = (v - val_l) / (val_h - val_l);
            drawNeedle(c, cx,cy, r1, r5, ratio, needleWidth5, needleColor1);
        }
    }
    // Scale 1
    if (val_s1 > 0.0) {
        for(v = val_l; v <= val_h; v += val_s1){
            ratio = (v - val_l) / (val_h - val_l);
            drawNeedle(c, cx,cy, r1, r4, ratio, needleWidth4, needleColor1);
        }
    }                                           // ADD END 2015.03.10
	// Maximum peak needle
	ratio = (val_peak1 - val_l) / (val_h - val_l);
    drawNeedle(c, cx,cy, r1, r2, ratio, needleWidth1, needleColor1);
    
	// Minimum peak needle
	ratio = (val_peak2 - val_l) / (val_h - val_l);
    drawNeedle(c, cx,cy, r1, r2, ratio, needleWidth2, needleColor2);
	
	// Main needle
	ratio = (val - val_l) / (val_h - val_l);
    drawNeedle(c, cx,cy, r1, r2, ratio, needleWidth3, needleColor3);
}

- (IBAction)setValue:(id)sender
{
    val_t = [sender floatValue];
	if (val_peak1 < val_t) {
		val_peak1 = val_t;
	}
	if (val_peak2 > val_t) {
		val_peak2 = val_t;
	}
}

- (IBAction)setIntValue:(int)sender
{
    val_t = sender;
	if (val_peak1 < val_t) {
		val_peak1 = val_t;
	}
	if (val_peak2 > val_t) {
		val_peak2 = val_t;
	}
}

- (IBAction)setFloatValue:(float)sender
{
    val_t = sender;
	if (val_peak1 < val_t) {
		val_peak1 = val_t;
	}
	if (val_peak2 > val_t) {
		val_peak2 = val_t;
	}
}

- (IBAction)setPeakValue:(id)sender
{
    val_peak1 = val_t;
    val_peak2 = val_t;
}

- (IBAction)refreshView:(id)sender
{
    self.val += ((self.val_t - self.val) / 5.0);
    [self setNeedsDisplay];
}

@end
