//
//  DetailViewController.m
//  WHM viewer iOS
//
//  Created by 富樫 豊彦 on 2016/03/08.
//  Copyright © 2016年 富樫 豊彦. All rights reserved.
//

#import "DetailViewController.h"

@interface DetailViewController ()
@property NSTimer *updateTimer;
@end

@implementation DetailViewController

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem {
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        // Update the view.
        [self configureView];
    }
}

- (void)configureView {
    // Update the user interface for the detail item.
    if (self.detailItem) {
        NSMutableArray *channels = self.detailItem;
        NSDictionary   *channel  = channels[self.detailNumber];
        float effectivePower = [[channel objectForKey:@"effectivePower"] floatValue];
        float voltage        = [[channel objectForKey:@"voltage"]        floatValue];
        float current        = [[channel objectForKey:@"current"]        floatValue];
        float apparentPower  = voltage * current;
        float powerFactor    = effectivePower / apparentPower * 100.0;
        float energyOfMonth  = [[channel objectForKey:@"energyOfMonth"] floatValue];
        NSDate *now = [NSDate dateWithTimeIntervalSinceNow:0.0f];
        NSDateFormatter *format = [[NSDateFormatter alloc] init];
        [format setDateFormat:@"yyyy/MM"];
        NSString *nowDate = [format stringFromDate:now];
        NSDate *dayOne = [format dateFromString:nowDate];
        NSTimeInterval since = [now timeIntervalSinceDate:dayOne];
        float energyOfGuess = (30*24*60*60) / (double)since * energyOfMonth;
        float averagePower = energyOfMonth / ((double)since / (60*60)) * 1000.0;
        float averageFactor =  effectivePower / averagePower;
        
        self.peripheralNameLabel.text = [[channel objectForKey:@"peripheral"] name];
        self.chLabel.text             = [NSString stringWithFormat:@"CH%d", [[channel objectForKey:@"ch"] intValue]];
        self.effectivePowerLabel.text = [NSString stringWithFormat:@"%.1f", effectivePower];
        self.energyOfMonthLabel.text  = [NSString stringWithFormat:@"%.2f", energyOfMonth];
        self.apparentPowerLabel.text  = [NSString stringWithFormat:@"%.1f", apparentPower];
        self.powerFactorLabel.text    = [NSString stringWithFormat:@"%.0f", powerFactor];
        self.voltageLabel.text        = [NSString stringWithFormat:@"%.1f", voltage];
        self.currentLabel.text        = [NSString stringWithFormat:@"%.2f", current];
        self.frequencyLabel.text      = [NSString stringWithFormat:@"%d",   [[channel objectForKey:@"frequency"] intValue]];
        self.energyOfGuessLabel.text  = [NSString stringWithFormat:@"%.2f", energyOfGuess];
        self.averagePowerLabel.text   = [NSString stringWithFormat:@"%.1f", averagePower];
        self.averageFactorLabel.text  = [NSString stringWithFormat:@"%.0f", averageFactor];
        [self.effectivePowerGauge  setFloatValue:effectivePower];
        [self.effectivePowerGauge2 setFloatValue:effectivePower];
        [self.effectivePowerGauge3 setFloatValue:effectivePower];
        [self.effectivePowerGauge4 setFloatValue:effectivePower];
        [self.powerFactorGauge     setFloatValue:powerFactor];
        [self.voltageGauge         setFloatValue:voltage];
        [self.currentGauge         setFloatValue:current];
        [self.averagePowerGauge    setFloatValue:averagePower];
        [self.averageFactorGauge   setFloatValue:averageFactor];
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    [self configureView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.updateTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(configureView) userInfo:nil repeats:YES];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.updateTimer invalidate];
}

@end
