//
//  LinerGauge.m
//  HA7 logger mini
//  Copyright(C)2012-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.02.08 v0.0  First cording
//    2012.02.12 v0.2  Commit
//    2013.04.27 v0.4  iOS
//    2015.03.10 v0.5  scale1 and 2
//
#import "LinerGauge.h"
#import <CoreGraphics/CoreGraphics.h>

@interface LinerGauge ()
@property (nonatomic)   CGColorRef  colorNormal;
@property (nonatomic)   CGColorRef  colorWarning;
@property (nonatomic)   CGColorRef  colorCritical;
@property (nonatomic)   CGColorRef  colorMinLine;
@property (nonatomic)   CGColorRef  colorMaxLine;
@end

@implementation LinerGauge
@synthesize val;
@synthesize val_l;
@synthesize val_h;
@synthesize val_w;
@synthesize val_c;
@synthesize val_peak1;
@synthesize val_peak2;
@synthesize val_s1;                         // ADD 2015.03.10
@synthesize val_s2;                         // ADD 2015.03.10
@synthesize val_low;
@synthesize val_high;
@synthesize val_warning;
@synthesize val_critical;
@synthesize val_scale1;                     // ADD 2015.03.10
@synthesize val_scale2;                     // ADD 2015.03.10

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
    }
    return self;
}

/*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect
 {
 // Drawing code
 }
 */


- (void)awakeFromNib
{
	CGFloat comps[4];

	val_h  = [val_high floatValue];
	val_l  = [val_low floatValue];
	val_w  = [val_warning floatValue];
	val_c  = [val_critical floatValue];
    val_s1 = [val_scale1 floatValue];       // ADD 2015.03.10
    val_s2 = [val_scale2 floatValue];       // ADD 2015.03.10
	val    = 0;
	val_peak1 = self.val;
	val_peak2 = self.val;

	CGColorSpaceRef rgb = CGColorSpaceCreateDeviceRGB();
    comps[0] = 1.0; comps[1] = 0.0; comps[2] = 0.0; comps[3] = 1.0;
    self.colorNormal = CGColorCreate(rgb, comps);
    comps[0] = 0.0; comps[1] = 1.0; comps[2] = 0.0; comps[3] = 1.0;
    self.colorWarning = CGColorCreate(rgb, comps);
    comps[0] = 0.0; comps[1] = 1.0; comps[2] = 1.0; comps[3] = 1.0;
    self.colorCritical = CGColorCreate(rgb, comps);
    comps[0] = 1.0; comps[1] = 1.0; comps[2] = 1.0; comps[3] = 1.0;
    self.colorMaxLine = CGColorCreate(rgb, comps);
    comps[0] = 1.0; comps[1] = 1.0; comps[2] = 1.0; comps[3] = 1.0;
    self.colorMinLine = CGColorCreate(rgb, comps);
	CGColorSpaceRelease(rgb);
}


//- (void)drawRect:(NSRect)rect
- (void)drawRect:(CGRect)rect
{
	float cx;
	float w;
	float h;
	float x1,y1;
	float x2,y2;
	float ratio;
	float zero;
    float v;                                    // ADD 2015.03.10
	CGContextRef    c = UIGraphicsGetCurrentContext();

//	cx = NSMidX(rect);
//	w  = NSWidth(rect);
//	h  = NSHeight(rect);
	cx = CGRectGetMidX(rect);
	w  = CGRectGetWidth(rect);
	h  = CGRectGetHeight(rect);
    
    // Scale 2                                  // ADD START 2015.03.10
    if (val_s2 > 0.0) {
        for(v = val_l; v <= val_h; v += val_s2){
            ratio = (v - val_l) / (val_h - val_l);
            x1 = 0;
            y1 = h - h * ratio;
            x2 = w * 0.2;
            y2 = y1;
            CGContextSetLineWidth(c, 0.5);
            CGContextSetStrokeColorWithColor(c, self.colorMaxLine);
            CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
            CGContextStrokePath(c);
        }
    }
    // Scale 1
    if (val_s1 > 0.0) {
        for(v = val_l; v <= val_h; v += val_s1){
            ratio = (v - val_l) / (val_h - val_l);
            x1 = 0;
            y1 = h - h * ratio;
            x2 = w * 0.7;
            y2 = y1;
            CGContextSetLineWidth(c, 0.5);
            CGContextSetStrokeColorWithColor(c, self.colorMaxLine);
            CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
            CGContextStrokePath(c);
        }
    }                                           // ADD END 2015.03.10
    
	// Maximum peak line
	ratio = (val_peak1 - val_l) / (val_h - val_l);
	x1 = 0;
//	y1 = h * ratio;
	y1 = h - h * ratio;
	x2 = w;
	y2 = y1;
//	[[NSColor greenColor] set];
//	[NSBezierPath setLineWidth:2.0];
//	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];
    CGContextSetLineWidth(c, 2.0);
    CGContextSetStrokeColorWithColor(c, self.colorMaxLine);
    CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
	CGContextStrokePath(c);
	
	// Minimum peak line
	ratio = (val_peak2 - val_l) / (val_h - val_l);
	x1 = 0;
//	y1 = h * ratio;
	y1 = h - h * ratio;
	x2 = w;
	y2 = y1;
//	[[NSColor greenColor] set];
//	[NSBezierPath setLineWidth:2.0];
//	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];
    CGContextSetLineWidth(c, 2.0);
    CGContextSetStrokeColorWithColor(c, self.colorMinLine);
    CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
	CGContextStrokePath(c);
	
	// Main bar
	ratio = (val - val_l) / (val_h - val_l);
	zero = -val_l / (val_h - val_l);
	x1 = cx;
//	y1 = h * zero;
	y1 = h - h * zero;
	x2 = cx;
//	y2 = h * ratio;
	y2 = h - h * ratio;
	if (val > val_c) {
//		[[NSColor redColor] set];
        CGContextSetStrokeColorWithColor(c, self.colorCritical);
	} else if (val > val_w) {
//		[[NSColor yellowColor] set];
        CGContextSetStrokeColorWithColor(c, self.colorWarning);
	} else {
//		[[NSColor greenColor] set];
        CGContextSetStrokeColorWithColor(c, self.colorNormal);
	}
//	[NSBezierPath setLineWidth:w];
//	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1)     toPoint:NSMakePoint(x2,y2)];
//	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1-3.0) toPoint:NSMakePoint(x1,y1+3.0)];
    CGContextSetLineWidth(c, w);
    CGContextMoveToPoint   (c, x1, y1);     CGContextAddLineToPoint(c, x2, y2);
    CGContextMoveToPoint   (c, x1, y1-3.0); CGContextAddLineToPoint(c, x1, y1+3.0);
	CGContextStrokePath(c);
}


- (IBAction)setValue:(id)sender
{
    val = [sender floatValue];
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
    
	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}


- (IBAction)setIntValue:(int)sender
{
    val = sender;
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
    
	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}


- (IBAction)setFloatValue:(float)sender
{
    val = sender;
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
    
	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}


- (IBAction)setPeakValue:(id)sender
{
    val_peak1 = val;
    val_peak2 = val;
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
}

                                    // ADD START 2016.03.19
- (IBAction)setWarnValue:(float)sender
{
    val_w = sender;
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
}


- (IBAction)setCriticalValue:(float)sender
{
    val_c = sender;
//  [self setNeedsDisplay:YES];
    [self setNeedsDisplay];
}
                                    // ADD END 2016.03.19

//- (void)mouseDown:(NSEvent *)theEvent
//{
//    val_peak1 = val;
//    val_peak2 = val;
//    [self setNeedsDisplay:YES];
//}

@end
