//
//  LinerGauge2.m
//  WHM viewer iOS
//
//  Created by 富樫 豊彦 on 2016/03/20.
//  Copyright © 2016年 富樫 豊彦. All rights reserved.
//

#import "LinerGauge2.h"
#import <CoreGraphics/CoreGraphics.h>

@implementation LinerGauge2

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (void)awakeFromNib {
    self.scaleHigh     = [self.confScaleHigh   floatValue];
    self.scaleLow      = [self.confScaleLow    floatValue];
    self.scaleSpan1    = [self.confScaleSpan1  floatValue];
    self.scaleSpan2    = [self.confScaleSpan2  floatValue];
    self.scaleLen1     = [self.confScaleLen1   floatValue];
    self.scaleLen2     = [self.confScaleLen2   floatValue];
    self.warnValue     = [self.confWarnValue   floatValue];
    self.criticalValue = [self.confCriticalValue floatValue];
    self.value         = 0;
    self.peakValue1    = 0;
    self.peakValue2    = 0;
    self.colorScale1   = [self.confColorScale1   CGColor];
    self.colorScale2   = [self.confColorScale2   CGColor];
    self.colorNormal   = [self.confColorNormal   CGColor];
    self.colorWarn     = [self.confColorWarn     CGColor];
    self.colorCritical = [self.confColorCritical CGColor];
    self.colorPeak1    = [self.confColorPeak1    CGColor];
    self.colorPeak2    = [self.confColorPeak2    CGColor];
}

- (void)drawRect:(CGRect)rect {
    float cx;
    float w;
    float h;
    float x1,y1;
    float x2,y2;
    float ratio;
    float zero;
    float v;
    CGContextRef    c = UIGraphicsGetCurrentContext();
    
    cx = CGRectGetMidX(rect);
    w  = CGRectGetWidth(rect);
    h  = CGRectGetHeight(rect);
    
    // Scale 2
    if (self.scaleSpan2 > 0.0) {
        for(v = self.scaleLow; v <= self.scaleHigh; v += self.scaleSpan2){
            ratio = (v - self.scaleLow) / (self.scaleHigh - self.scaleLow);
            x1 = 0;
            y1 = h - h * ratio;
            x2 = w * self.scaleLen2;
            y2 = y1;
            CGContextSetLineWidth(c, 0.5);
            CGContextSetStrokeColorWithColor(c, self.colorScale2);
            CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
            CGContextStrokePath(c);
        }
    }
    // Scale 1
    if (self.scaleSpan1 > 0.0) {
        for(v = self.scaleLow; v <= self.scaleHigh; v += self.scaleSpan1){
            ratio = (v - self.scaleLow) / (self.scaleHigh - self.scaleLow);
            x1 = 0;
            y1 = h - h * ratio;
            x2 = w * self.scaleLen1;
            y2 = y1;
            CGContextSetLineWidth(c, 0.5);
            CGContextSetStrokeColorWithColor(c, self.colorScale1);
            CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
            CGContextStrokePath(c);
        }
    }
    
    // Maximum peak line
    ratio = (self.peakValue1 - self.scaleLow) / (self.scaleHigh - self.scaleLow);
    x1 = 0;
//	y1 = h * ratio;
    y1 = h - h * ratio;
    x2 = w;
    y2 = y1;
    CGContextSetLineWidth(c, 2.0);
    CGContextSetStrokeColorWithColor(c, self.colorPeak1);
    CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
    CGContextStrokePath(c);
    
    // Minimum peak line
    ratio = (self.peakValue2 - self.scaleLow) / (self.scaleHigh - self.scaleLow);
    x1 = 0;
//	y1 = h * ratio;
    y1 = h - h * ratio;
    x2 = w;
    y2 = y1;
    CGContextSetLineWidth(c, 2.0);
    CGContextSetStrokeColorWithColor(c, self.colorPeak2);
    CGContextMoveToPoint(c, x1, y1);    CGContextAddLineToPoint(c, x2, y2);
    CGContextStrokePath(c);
    
    // Main bar
    ratio = (self.value - self.scaleLow) / (self.scaleHigh - self.scaleLow);
    zero = -self.scaleLow / (self.scaleHigh - self.scaleLow);
    x1 = cx;
//	y1 = h * zero;
    y1 = h - h * zero;
    x2 = cx;
//	y2 = h * ratio;
    y2 = h - h * ratio;
    if (self.value > self.criticalValue) {
        CGContextSetStrokeColorWithColor(c, self.colorCritical);
    } else if (self.value > self.warnValue) {
        CGContextSetStrokeColorWithColor(c, self.colorWarn);
    } else {
        CGContextSetStrokeColorWithColor(c, self.colorNormal);
    }
    CGContextSetLineWidth(c, w);
    CGContextMoveToPoint   (c, x1, y1);     CGContextAddLineToPoint(c, x2, y2);
    CGContextMoveToPoint   (c, x1, y1-3.0); CGContextAddLineToPoint(c, x1, y1+3.0);
    CGContextStrokePath(c);
}

- (IBAction)setFloatValue:(float)sender {
    self.value = sender;
    if (self.value > self.scaleHigh) {
        self.value = self.scaleHigh;
    }
    if (self.value < self.scaleLow) {
        self.value = self.scaleLow;
    }
    if (self.peakValue1 < self.value) {
        self.peakValue1 = self.value;
    }
    if (self.peakValue2 > self.value) {
        self.peakValue2 = self.value;
    }
    [self setNeedsDisplay];
}

- (IBAction)setPeakValue:(id)sender {
    self.peakValue1 = self.value;
    self.peakValue2 = self.value;
    [self setNeedsDisplay];
}

@end
