//
//  MasterViewController.m
//  WHM viewer iOS
//
//  Created by 富樫 豊彦 on 2016/03/08.
//  Copyright © 2016年 富樫 豊彦. All rights reserved.
//

#import "MasterViewController.h"
#import "TableViewCell.h"
#import "DetailViewController.h"
#import <CoreBluetooth/CoreBluetooth.h>

#define SERVICE_UUID           @"B61CF193-79B3-450A-BE2F-16D46E216C00"
#define CHARACTERISTIC_UUID0   @"B61CF193-79B3-450A-BE2F-16D46E216D00"
#define CHARACTERISTIC_UUID1   @"B61CF193-79B3-450A-BE2F-16D46E216D01"
#define CHARACTERISTIC_UUID2   @"B61CF193-79B3-450A-BE2F-16D46E216D02"
#define CHARACTERISTIC_UUID3   @"B61CF193-79B3-450A-BE2F-16D46E216D03"
#define CHARACTERISTIC_UUID4   @"B61CF193-79B3-450A-BE2F-16D46E216D04"
#define CHARACTERISTIC_UUID5   @"B61CF193-79B3-450A-BE2F-16D46E216D05"
#define CHARACTERISTIC_UUID6   @"B61CF193-79B3-450A-BE2F-16D46E216D06"
#define CHARACTERISTIC_UUID7   @"B61CF193-79B3-450A-BE2F-16D46E216D07"
#define CHARACTERISTIC_UUID8   @"B61CF193-79B3-450A-BE2F-16D46E216D08"

@interface MasterViewController () <CBCentralManagerDelegate, CBPeripheralDelegate>
@property (strong, nonatomic) CBCentralManager      *centralManager;
@property NSMutableArray *whm;
@end

@implementation MasterViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    //  self.navigationItem.leftBarButtonItem = self.editButtonItem;
    self.detailViewController = (DetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
    
    NSLog(@"BT Init");
    self.centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
    self.whm = [[NSMutableArray alloc] init];
}

- (void)viewWillAppear:(BOOL)animated {
    self.clearsSelectionOnViewWillAppear = self.splitViewController.isCollapsed;
    [super viewWillAppear:animated];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Segues

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([[segue identifier] isEqualToString:@"showDetail"]) {
        NSIndexPath    *indexPath  = [self.tableView indexPathForSelectedRow];
        CBPeripheral   *peripheral = [self.whm[indexPath.section] objectForKey:@"peripheral"];
        NSMutableArray *channels   = [self.whm[indexPath.section] objectForKey:@"channel"];
        NSDictionary   *object     = channels[indexPath.row];
        
        DetailViewController *controller = (DetailViewController *)[[segue destinationViewController] topViewController];
        //      [controller setDetailItem:object];
        [controller setDetailItem:channels];
        [controller setDetailNumber:indexPath.row];
        controller.navigationItem.leftBarButtonItem = self.splitViewController.displayModeButtonItem;
        controller.navigationItem.leftItemsSupplementBackButton = YES;
        controller.navigationItem.title = [NSString stringWithFormat:@"%@ - CH%@", peripheral.name, [object objectForKey:@"ch"]];
    }
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.whm.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [[self.whm[section] objectForKey:@"channel"] count];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    CBPeripheral *peripheral = [self.whm[section] objectForKey:@"peripheral"];
    return peripheral.name;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    TableViewCell  *cell       = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    CBPeripheral   *peripheral = [self.whm[indexPath.section] objectForKey:@"peripheral"];
    NSMutableArray *channels   = [self.whm[indexPath.section] objectForKey:@"channel"];
    NSDictionary   *object     = channels[indexPath.row];
    
    NSNumber *ch              = [object objectForKey:@"ch"];
    NSNumber *effectivePower  = [object objectForKey:@"effectivePower"];
    NSNumber *energyOfMonth   = [object objectForKey:@"energyOfMonth"];
    cell.PeriperalLabel.text  = peripheral.name;
    cell.ChannelLabel.text    = [NSString stringWithFormat:@"CH%d",    [ch intValue]];
    cell.PowerLabel.text      = [NSString stringWithFormat:@"%dW",     [effectivePower intValue]];
    cell.EnergyLabel.text     = [NSString stringWithFormat:@"%.2fkWh", [energyOfMonth floatValue]];
    return cell;
}

#ifdef DEL
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return NO;
}
#endif

#pragma mark - Bluetooth-LE central

- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    NSLog(@"BT update %ld", (long)central.state);
    if (central.state != CBCentralManagerStatePoweredOn) {
        return;
    }
    [self scan];
}


- (void)scan {
    [self.centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:SERVICE_UUID]] options:nil];
    NSLog(@"Scanning...");
}


- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    int i;
    
    NSLog(@"Discovered %@ at %@db", peripheral.name, RSSI);
    for(i = 0; i < [self.whm count]; i++) {
        CBPeripheral *p = [self.whm[i] objectForKey:@"peripheral"];
        if ([p.identifier isEqual:peripheral.identifier]) {
            break;
        }
    }
    if (i >= [self.whm count]) {
        //      NSLog(@"New entry %@ %@", peripheral.name, peripheral.identifier);
        NSMutableArray *channel = [[NSMutableArray alloc] init];
        NSDictionary *whm =[NSDictionary dictionaryWithObjectsAndKeys:
                            peripheral, @"peripheral",
                            channel, @"channel",
                            nil];
        [self.whm insertObject:whm atIndex:0];
        NSLog(@"Connecting.... %@", peripheral);
        [self.centralManager connectPeripheral:peripheral options:nil];
    }
}

- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    NSLog(@"Failed to connect to %@. (%@)", peripheral, [error localizedDescription]);
    [self cleanup];
}


- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
    NSLog(@"Connected %@", peripheral);
    [self.centralManager stopScan];
    peripheral.delegate = self;
    [peripheral discoverServices:@[[CBUUID UUIDWithString:SERVICE_UUID]]];
}


#pragma mark - Bluetooth-LE peripheral

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {
    if (error) {
        NSLog(@"Error discovering services: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBService *service in peripheral.services) {
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID1]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID2]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID3]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID4]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID5]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID6]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID7]] forService:service];
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
    if (error) {
        NSLog(@"Error discovering characteristics: set notify: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBCharacteristic *characteristic in service.characteristics) {
        if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]]) {   // Date time
            struct {
                short yyyy;
                char mm;
                char dd;
                char hh;
                char mi;
                char ss;
            } val;
            struct tm tm;
            time_t  t;
            
            t = time(&t);
            localtime_r(&t, &tm);
            val.yyyy = tm.tm_year + 1900;
            val.mm   = tm.tm_mon + 1;
            val.dd   = tm.tm_mday;
            val.hh   = tm.tm_hour;
            val.mi   = tm.tm_min;
            val.ss   = tm.tm_sec;
            NSData *val1 = [[NSData alloc] initWithBytes:&val length:sizeof(val)];
            [peripheral  writeValue:val1 forCharacteristic:characteristic type:CBCharacteristicWriteWithResponse];
            NSLog(@"Send date time %@", characteristic);
        } else {
            [peripheral setNotifyValue:YES forCharacteristic:characteristic];
            //          NSLog(@"Set notify %@", characteristic);
        }
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    int     channelNumber;
    
    if (error) {
        NSLog(@"Error discovering characteristics: update value: %@", [error localizedDescription]);
        return;
    }
    
    channelNumber = 0;
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]]) {
        struct tm tm;
        time_t t;
        
        tm.tm_year = *((short *)[characteristic.value bytes])    - 1900;        // Byte order is Little endian
        tm.tm_mon  = *((char *)[characteristic.value bytes] + 2) - 1;
        tm.tm_mday = *((char *)[characteristic.value bytes] + 3);
        tm.tm_hour = *((char *)[characteristic.value bytes] + 4);
        tm.tm_min  = *((char *)[characteristic.value bytes] + 5);
        tm.tm_sec  = *((char *)[characteristic.value bytes] + 6);
        t = mktime(&tm);
        //      self.recvieveTime = [[NSDate alloc] initWithTimeIntervalSinceNow:0];
        //      self.dataTime = [[NSDate alloc] initWithTimeIntervalSince1970:t];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID1]]) {
        channelNumber = 1;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID2]]) {
        channelNumber = 2;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID3]]) {
        channelNumber = 3;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID4]]) {
        channelNumber = 4;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID5]]) {
        channelNumber = 5;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID6]]) {
        channelNumber = 6;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID7]]) {
        channelNumber = 7;
    } else {
        channelNumber = 8;
    }
    if (channelNumber > 0) {
        int     i,j;
        float   power;
        float   current;
        float   voltage;
        float   frequency;
        float   energyOfMonth;
        
        // Set data
        power         = *((float *)[characteristic.value bytes]);
        current       = *((float *)[characteristic.value bytes] + 1);
        voltage       = *((float *)[characteristic.value bytes] + 2);
        frequency     = *((float *)[characteristic.value bytes] + 3);
        energyOfMonth = *((float *)[characteristic.value bytes] + 4) / 1000.0;
        
        NSDictionary *newRecord = [NSDictionary dictionaryWithObjectsAndKeys:
                                   peripheral,                               @"peripheral",
                                   [NSNumber numberWithInt:channelNumber],   @"ch",
                                   [NSNumber numberWithFloat:power],         @"effectivePower",
                                   [NSNumber numberWithFloat:current],       @"current",
                                   [NSNumber numberWithFloat:voltage],       @"voltage",
                                   [NSNumber numberWithFloat:frequency],     @"frequency",
                                   [NSNumber numberWithFloat:energyOfMonth], @"energyOfMonth",
                                   nil];
        i = channelNumber - 1;
        for(j = 0; j < [self.whm count]; j++) {
            CBPeripheral *p = [self.whm[j] objectForKey:@"peripheral"];
            if (p == peripheral) {
                break;
            }
        }
        if (j < [self.whm count]) {
            NSMutableArray *channel = [self.whm[j] objectForKey:@"channel"];
            if (channel.count > i) {
                [channel replaceObjectAtIndex:i withObject:newRecord];
            } else {
                if (channel.count < i) {
                    [channel insertObject:newRecord atIndex:0];
                } else {
                    [channel insertObject:newRecord atIndex:i];
                }
            }
        }
        [self.tableView reloadData];
    }
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    int i;
    
    NSLog(@"Peripheral Disconnected");
    for(i = 0; i < [self.whm count]; i++) {
        CBPeripheral *p = [self.whm[i] valueForKeyPath:@"peripheral"];
        if (p == peripheral) {
            [self.whm removeObjectAtIndex:i];
        }
    }
    [self scan];
}


- (void)cleanup {
    NSLog(@"BT Cleanup");
    return;
}

@end
